/**************************************************************/
/*                                                            */
/*   Program Name :  datacut                                  */
/*                                                            */
/*      Version:  0.00  2003-09-24                            */
/*                                                            */
/*      Copyright (c) 2003 T.Kondo/CRL All Right Reserved     */
/*                                                            */
/**************************************************************/
/*
 ho|ukah{[h擾f[^t@C̎Ԕ͈͂
 f[^𒊏o

          T.Kondo 2003.9.26
               V0.0 2003.9.24

    点@datacut filename [soffset [period [outfile]]]
          filename -- f[^t@C
                            0 Ƃƃt@C
                      @@@tds.datag
               soffset -- X^[gItZbgibj
                           ftHgl0
               period  -- of[^̎Ԓibj
                           ftHgl10
               outfile -- f[^o̓t@C
                           ftHgdatacut.dat


*/
#include "chead.h"      // common header for IP-VLBI utilities

#include "libipvlbi.h"  // IP-VLBIp֐Q(by T.Kondo)

#ifndef PIRA
#define PIRA  1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef PIDE
#define PIDE  57.29577951       // 180/PI radian to degree
#endif
#ifndef TWOPI
#define TWOPI 6.283185307       // 2PI
#endif
#ifndef PI
#define PI    3.141592653
#endif


typedef struct ip_vlbi_file
{
	unsigned char bit32[4];
	unsigned char bit64[8];
	//char buf[65536];
	int sfreq;
	int icnt;   //  current position number
	char* fname; //I[vĂt@C
	float fsample;  //TvOg(Hz)
	int   adbit;  // ADrbg
	long int seconds; // 00h00m00šoߕb
	int hh, mm, ss;
	int spatn; // sync pattern
	int syncok; // =1 for 32bit all 1 detection
	int format; // =0 for old format , =1 for new format
	FILE *stream;
	fpos_t pos;
	int fh;
	int ch;
	int numch;
	int ierr;
	double dtime;
	unsigned int bytesread;
} ip_vlbi_file_t;

int checkheader(ip_vlbi_file_t *S)
{
       /* Read 64bit Data header check to get sampling parameters */
       int iok, i;

       S->bytesread = fread( S->bit64, sizeof( char ), 8, S->stream );
       //printf("bytes read 1 %d\n",bytesread); //debug

       if( ferror( S->stream ) || S->bytesread==0 )
       {
           printf("checkheader: File EOF or Read Error!\n");
	       i=fclose( S->stream ) ;
		   return -1;
	   }


       headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
                        &S->seconds,&S->hh,&S->mm,&S->ss);
       if (iok == 0)   // sync detection
	   {
          printf("New Format (header 64bits) Sync Detected!!\n");
		  printf("File : %s\n",S->fname);
          printf("  A/D(bits) = %d  # of ch = %d  Sampling Freq(kHz) = %d\n",
                    S->adbit,S->numch, S->sfreq);
          printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
                    S->hh,S->mm,S->ss,S->seconds);
          S->icnt=0;          
          return 0;
	   }
       else
       {
           printf("First 64bit is not header block!\n");
	       i=fclose( S->stream ) ;
           return -1;
       }
}

int fwdNsec(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
	//printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
		        return -1;
            }
		}
        if(checkheader(&(*dat)) < 0){
             return -1;
        }
     }
     return 0;
}



int main(int argc, char *argv[])
{
       int sfreq, adbit, numch;
       long int smpl_count;    // sample number counts to extract
       long int counts;        //sample counter
       long int counts2;        //extract sample counter
       int nloop;
       int i,imax, m;
       int smode,usampl;
       unsigned char *ibuf;
       int nbyte=5000;
       int bytesread;
       int bits, bytes, numb;
	   int nch;
       double fsampl,tsampl,fact;
       int *idata;
	   //double soffset, fsec, period;
       int soffset,period;
       int nwrite;
       char fname[80],oname[80];
       char lab2[60], ltime1st[60];
       char labfsampl[10];
       char* dflt_fname="tds.data";
       char* dflt_oname="datacut.dat";
       ip_vlbi_file_t X;
       FILE *fo;

       if(argc == 1){
          printf("datacut Ver 0.0\n");
          printf("Usage: datacut filename [period [soffset [outfile]]]\n");
          printf("\n");
          printf("         where  filename ---- Data file name\n");
          printf("                             if 0 is given, tds.data will be used\n");
          printf("                soffset --- Start time offset in sec (integer seconds)\n");
          printf("                             default is 0\n");         
          printf("                period  --- Extracting period in sec (integer seconds)\n");
          printf("                             default is 10\n");         
          printf("                outfile ---- Extract data outfile name\n");
          printf("                             default is datacut.dat\n");         
          exit(0);
       }

       /* data file name get */
       strcpy(fname,*(argv+1));
       if (fname[0]=='0'){
          strcpy(fname,dflt_fname);
       }
       X.fname=fname;

       /* default set */
       soffset=0;     
       period=10;     
       strcpy(oname,dflt_oname);

       /* Start time offset get */
       if (argc>=3) soffset = atoi(*(argv + 2 ));

       /* Period to extract get */
       if (argc>=4) period = atoi(*(argv + 3 ));

       /* Out file name get */
       if (argc>=5) strcpy(oname,*(argv+4));

       /* Run condition monitor for debug*/
       //printf("Data File is %s\n",fname);
       //printf("nch, soffset, period  %d %f %f\n",nch,soffset,period);

       /* Program Name Display */
       printf("****************************************************\n");
       printf("*    IP-VLBI DATA TRUNCATION PROGRAM (datacut)     *\n");
       printf("*      Ver 0.0   2003-09-24  by  T.KONDO/CRL       *\n");
       printf("****************************************************\n");
       printf("\n");

       /* Data file open */
       if( (X.stream  = fopen( X.fname, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",fname);
		   goto exitend2;
       }

       /* output file create */
       if( (fo  = fopen( oname, "wb" )) == NULL )
       {
          printf("File  %s  Open Error! \n",oname);
          goto exitend3;
	   } 

       /* Data header check to get sampling parameters */
       if(checkheader(&X) < 0){
            goto exitend3;
       }

       /* parameter set */
        sfreq=X.sfreq;
        adbit=X.adbit;
        numch=X.numch;
        fsampl=1000.0*(float)sfreq;   // sampling frequency (Hz)
        tsampl=1.0/fsampl;            // sampling period (sec)
        bits=((long)fsampl)*adbit*numch;  // # of bits in 1 sec data except header
        bytes=bits/8;                     // # of bytes in 1 sec data except header
        X.fsample=(float)fsampl;

        /* Set extract Ch# to 1 in case of 1 ch board*/
        if(numch == 1){nch=1; }   


        /* File read buffer size set */
        smode=1;                         // low resolution mode

        if(smode ==1){ usampl=10000; }
        else {
           usampl=50000;
           if(sfreq == 40){ usampl=40000; }
           if(sfreq == 100){ usampl=100000; }
           if(sfreq == 200){ usampl=200000; }
        }
        imax=(sfreq*1000)/usampl;
        numb=usampl*adbit*numch/8;   // # of bytes in unit # of samples

        //printf("imax numb  %d %d\n",imax,numb);    //debug


        ibuf = (unsigned char *)malloc(numb);
        idata = (int *)malloc(4*usampl*sizeof(int));

        fact=((float)usampl)*((float)usampl);
        
        /* sampling frequency in text for print */
        if(sfreq == 40){strcpy(labfsampl,"40kHz"); }
        if(sfreq == 100){strcpy(labfsampl,"100kHz"); }
        if(sfreq == 200){strcpy(labfsampl,"200kHz"); }
        if(sfreq == 500){strcpy(labfsampl,"500kHz"); }
        if(sfreq == 1000){strcpy(labfsampl,"1MHz"); }
        if(sfreq == 2000){strcpy(labfsampl,"2MHz"); }
        if(sfreq == 4000){strcpy(labfsampl,"4MHz"); }
        if(sfreq == 8000){strcpy(labfsampl,"8MHz"); }
        if(sfreq == 16000){strcpy(labfsampl,"16MHz"); }

       /* Data extract start point set */ 
       //startoffset=(long int)soffset;
       //fsec=soffset-startoffset;
       //sofst_pos=(long int)(fsec/tsampl + 0.01);

       /* Data extraction period set */
       smpl_count=(long int)(period/tsampl + 0.01);

       /* Run condition display */

       //printf("startoffset, sofst_pos, smpl_count %d %d %d\n",startoffset,sofst_pos,smpl_count);
       printf("\n");
       printf("  Data File : %s\n",fname);
       printf("  Sampling Freq : %s\n",labfsampl);
       printf("  A/D bits      : %d\n",adbit);
       printf("  Number of CHs : %d\n",numch);

       /* foward startffset seconds */
       if(fwdNsec(ibuf, soffset, numb, imax, &X) <0){
              goto exitend;
       }
       printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d\0",X.hh,X.mm,X.ss);
       strcpy(lab2,ltime1st);
        
       /* continue run info display */
       printf("  Start Offset (sec)       : %d\n",soffset);
       printf("  Start Time of Extraction : %02d:%02d:%02d\n",X.hh,X.mm,X.ss);
       printf("  Period (sec)             : %d\n",period);
       printf("  Out File : %s\n",oname);


       /* Data extraction start */
	   counts=0;
	   counts2=0;
       m=nch-1;
       nloop=0;

       /* 1st header copy */
       nwrite = fwrite( X.bit64, sizeof( char ), 8, fo);


       /* Data read loop start here */

loop1:
        if(nloop >0) {
            if(nloop >= period) goto exithere;
           if(checkheader(&X) < 0) goto exitend; 
            //printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
            nwrite = fwrite( X.bit64, sizeof( char ), 8, fo);
        }

        for(i=0; i< imax; i++)
        {
             bytesread = fread( ibuf, sizeof( char ), numb, X.stream );
             //printf("bytes read 2 %d\n",bytesread); //debug
             nwrite = fwrite( ibuf, sizeof( char ), numb, fo);
        }
        nloop++;
        goto loop1;
exithere:
       printf("\n");
       printf("Data truncation finished.\n");
       i=fclose( fo ) ;

exitend: 
       
       free(ibuf);
       free(idata);

exitend3: 
       i=fclose( X.stream ) ;

 exitend2: 

       return 0;
}
